;                                                                               
;              |                                          |                     
;          ____|      ____________________________________|_____                
;              |     |     |                              |                     
;  "1" SLOT    |  6  |     9       64                     |                     
;              |_____|                                    |                     
;              |                                       	  |                     
;              |           Samples                     	  |                     
;              |                                       	  |                     
;          ____|                                    ______|_____                
;              |           15                      |      |                     
; "0" SLOT     |           |        60             |  10  |                     
;              |___________________________________|      |                     
;              |                                       	  |                     
;              |                                          |                     
;                                                                               
;                                                      	                        
;          ____                    ______          _______________|             
;              |                  |      |        |               |             
; RESET        |                  |      | Samples|               |             
;              |__________________|      |____|___|               |             
;               <------480--------><---70----><--------410------->|             
;                                                                 |             
;                                                                               
; ##############################################################################
; ##############################################################################
; ##############################################################################
; 																				
; EICRA  External Interrupt Control Register A									
; ISCn1 ISCn0 Description														
; 	0 	0 	The low level of INTn generates an interrupt request.				
; 	0 	1 	Any edge of INTn generates asynchronously an interrupt request.		
; 	1 	0 	The falling edge of INTn generates asynchronously an interrupt request.
; 	1 	1 	The rising edge of INTn generates asynchronously an interrupt request.
; 																				
EXT_INTERRUPT_INITIALISIERUNG:
	; PULL-UP-ON									
	;sbi PORTD,2			; INT0--> PD2	
	;sbi PORTD,PD3		; INT1--> PD3	
	;sbi PORTB,PB2		; INT2--> PB2	
	; EICRA External Interrupt Control Register A	
	ldi temp, (0<<ISC21)|(1<<ISC20)|(0<<ISC11)|(1<<ISC10)|(0<<ISC01)|(1<<ISC00)
	STS EICRA,temp
	; EIMSK  External Interrupt Mask Register		
	ldi temp, (0<<INT2)|(0<<INT1)|(0<<INT0)
	OUT EIMSK,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
EXT_INT0:
	LED_C_ON
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
EXT_INT1:
	LED_C_ON
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
; PIN4 vom ENC																	
EXT_INT2:
	LED_C_ON
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Pin Change Mask Register														
; PCMSK0: PCINT7  PCINT6  PCINT5  PCINT4 *PCINT3  PCINT2  PCINT1  PCINT0  (PA)	
; PCMSK1: PCINT15 PCINT14 PCINT13 PCINT12 PCINT11 PCINT10 PCINT9  PCINT8  (PB)	
; PCMSK2: PCINT23 PCINT22 PCINT21 PCINT20 PCINT19 PCINT18 PCINT17 PCINT16 (PC)	
; PCMSK3: PCINT31 PCINT30 PCINT29 PCINT28 PCINT27 PCINT26 PCINT25 PCINT24 (PD)	
;																				
; Pin Change Interrupt Control Register											
; PCICR - PCIE3 PCIE2 PCIE1 PCIE0												
PIN_CHANGE_INITIALISIERUNG:
	ldi temp,(0<<PCINT7)|(0<<PCINT6)|(0<<PCINT5)|(0<<PCINT4)|(1<<PCINT3)|(0<<PCINT2)|(0<<PCINT1)|(0<<PCINT0)
	STS(PCMSK0),temp	; Pin Change Mask Register					

	ldi temp,(0<<PCINT15)|(0<<PCINT14)|(0<<PCINT13)|(0<<PCINT12)|(0<<PCINT11)|(0<<PCINT10)|(0<<PCINT9)|(0<<PCINT8)
	STS(PCMSK1),temp	; Pin Change Mask Register					


	ldi temp,(0<<PCIE3)|(0<<PCIE2)|(0<<PCIE1)|(1<<PCIE0)
	STS(PCICR),temp		; Pin Change Interrupt Control Register		
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
PIN_CHANGE_AUS:
	STS(PCMSK0),NULL
	STS(PCMSK1),NULL
	STS(PCICR),NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; fr 1WIRE																		
PCINTERRUPT0:
	SBIC(OWIRE_PIN),(OWIRE_PIN_NR)	; sofort zurck bei LOW-->HIGH Flanke, also wenn PIN HIGH ist	
	reti
;-------------------------------------------------------------------------------
PCINTERRUPT0_RUN:
	push TEMP		    ; Sichern von "TEMP" im Stack
	in	 TEMP,SREG		; Einlesen des SREG 
	push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)
	push TEMP1
	push TEMP2
	push TEMP3
	push ZL
	push ZH
;-------------------------------------------------------------------------------
	call OWIRE
;-------------------------------------------------------------------------------
	pop ZH
	pop ZL
	pop TEMP3
	pop TEMP2
	pop TEMP1
	pop	TEMP			; LESEN von SREG vom STACK (KOPIE)
	out	SREG,TEMP		; Wiederherstellen von SREG
	pop	TEMP			; Wiederherstellen von "TEMP" 
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
; PA3=PCINT3																	
OW_INITIALISIERUNG:
	cbi (OWIRE_DDR) ,(OWIRE_PIN_NR)			; auf Eingang	
	sbi (OWIRE_PORT),(OWIRE_PIN_NR)			; PULL-UP-ON	

	call PIN_CHANGE_INITIALISIERUNG
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; STATUS:	0=auf RP warten														
;			1=RX																
;																				
OWIRE:
	LDS temp,(adr_1WIRE_STATUS)
	tst temp
	brne OWIRE_SAMPLES 
	jmp OW_PRESENCE_CHECK				; RP-Check				
;-------------------------------------------------------------------------------
OWIRE_SAMPLES:
	; SAMPLES				
	call WAIT_20us
	CLC									; Carry clear			
	SBIC(OWIRE_PIN),(OWIRE_PIN_NR)		; Sprung, wenn PIN LOW	
	SEC									; Carry clear			
	; BIT hineinschieben	
	LDS temp,(adr_1WIRE_BYTE)
	ROR temp							; ROR rechts ber Carry	
	STS(adr_1WIRE_BYTE),temp
	; Bit-COUNTER+1			
	LDS temp,(adr_1WIRE_BIT_COUNTER)
	inc temp
	andi temp,0b00000111				; 0...7					
	STS(adr_1WIRE_BIT_COUNTER),temp
	tst temp
	breq OWIRE_SAMPLES_w
	rjmp OW_PRESENCE_CHECK				; RP-Check				
;-------------------------------------------------------------------------------
OWIRE_SAMPLES_w:
	; SAVE im SRAM			
	ldi ZL,LOW (adr_1WIRE_DATA)
	ldi ZH,HIGH(adr_1WIRE_DATA)
	LDS temp,(adr_1WIRE_BYTE_COUNTER)
	add ZL,temp
	adc ZH,NULL
	LDS temp,(adr_1WIRE_BYTE)
	ST Z,temp
	; AUSWERTUNG ?			
	LDS temp,(adr_1WIRE_BYTE_COUNTER)	; Counter	
	cpi temp,2+99
	breq OWIRE_SAMPLES_AUSWERTUNG		; ---> Auswertung
	; BYTE-COUNTER+1		
	LDS temp,(adr_1WIRE_BYTE_COUNTER)	; Counter	
	inc temp
	STS(adr_1WIRE_BYTE_COUNTER),temp	; Counter	
	; SKIP-ROM ?			
	cpi temp,2							; Counter	
	breq OWIRE_SAMPLES_w2
	rjmp OW_PRESENCE_CHECK				; RP-Check				
;-------------------------------------------------------------------------------
OWIRE_SAMPLES_w2:
	LDS temp,(adr_1WIRE_DATA+0)
	cpi temp,(OW_COMMAND_SKIP_ROM)
	brne OWIRE_SAMPLES_ERROR
	; WRITE_PRIVATE_DATA ?	
	LDS temp,(adr_1WIRE_DATA+1)
	cpi temp,(OW_FUNCTION_WRITE_PRIVATE_DATA)
	brne OWIRE_SAMPLES_ERROR
	rjmp OW_PRESENCE_CHECK				; RP-Check				
;-------------------------------------------------------------------------------
OWIRE_SAMPLES_ERROR:
	STS(adr_1WIRE_STATUS),NULL
	rjmp OWIRE_RX_SPERREN				; RX sperren	
; ##############################################################################
; ##############################################################################
; ##############################################################################
OWIRE_SAMPLES_AUSWERTUNG:
	LDS temp1,(adr_1WIRE_COUNTER_L)		; Counter					
	LDS temp2,(adr_1WIRE_COUNTER_H)
	add temp1,EINS
	adc temp2,NULL
	STS(adr_1WIRE_COUNTER_L),temp1
	STS(adr_1WIRE_COUNTER_H),temp2
	; Prfsumme					
	clr temp1		; Prfsumme	
	ldi ZL,LOW (adr_1WIRE_DATA+2)
	ldi ZH,HIGH(adr_1WIRE_DATA+2)
	ldi temp3,99	; Schleife	
OWIRE_SAMPLES_AUSWERTUNG_s:	
	LD temp,Z+
	add temp1,temp	; Prfsumme	
	dec temp3
	brne OWIRE_SAMPLES_AUSWERTUNG_s
	STS(adr_1WIRE_PS_BERECHNET),temp1
	; Vergleich					
	LDS temp,(adr_1WIRE_DATA+2+99)
	STS(adr_1WIRE_PS_EMPFANGEN),temp
	cp temp,temp1
	breq OWIRE_SAMPLES_AUSWERTUNG_OK
;-------------------------------------------------------------------------------
; ERROR																			
	STS(adr_LED_1WIRE_TIMER),NULL
	LDS temp1,(adr_1WIRE_PS_ERROR_L)
	LDS temp2,(adr_1WIRE_PS_ERROR_H)
	add temp1,EINS
	adc temp2,NULL
	STS(adr_1WIRE_PS_ERROR_L),temp1
	STS(adr_1WIRE_PS_ERROR_H),temp2
	rjmp OWIRE_RX_SPERREN					; RX sperren	
;-------------------------------------------------------------------------------
OWIRE_SAMPLES_AUSWERTUNG_OK:
	ldi temp,(ZEIT_LED_1WIRE)
	STS(adr_LED_1WIRE_TIMER),temp

	LDS temp,(adr_1WIRE_DATA+2+0) 		; Sekunden					
	STS(adr_ss),temp
	LDS temp,(adr_1WIRE_DATA+2+1) 		; Minuten					
	STS(adr_mm),temp
	LDS temp,(adr_1WIRE_DATA+2+2) 		; Stunden					
	STS(adr_hh),temp
	LDS temp,(adr_1WIRE_DATA+2+3) 		; TAG						
	STS(adr_TAG)  ,temp
	LDS temp,(adr_1WIRE_DATA+2+4) 		; Wochentag (1=Mo 7=So)		
	STS(adr_WOCHENTAG),temp
	LDS temp,(adr_1WIRE_DATA+2+5) 		; Monat						
	STS(adr_MONAT),temp
	LDS temp,(adr_1WIRE_DATA+2+6) 		; JAHR						
	STS(adr_JAHR) ,temp
	LDS temp,(adr_1WIRE_DATA+2+7) 		; SOMMERZEIT 0=MEZ 1=MESZ	
	STS(adr_MESZ) ,temp
	LDS temp,(adr_1WIRE_DATA+2+8) 		; SONNENAUFGANG	mm			
	STS(adr_SONNE_AUFGANG_mm),temp
	LDS temp,(adr_1WIRE_DATA+2+9)		; SONNENAUFGANG	hh			
	STS(adr_SONNE_AUFGANG_hh),temp
	LDS temp,(adr_1WIRE_DATA+2+10)		; SONNENUNTERGANG mm		
	STS(adr_SONNE_UNTERGANG_mm),temp
	LDS temp,(adr_1WIRE_DATA+2+11)		; SONNENUNTERGANG hh		
	STS(adr_SONNE_UNTERGANG_hh),temp
	LDS temp,(adr_1WIRE_DATA+2+12)		; Luftdruck digital			
	STS(adr_DRUCK_DIGITAL_L),temp
	LDS temp,(adr_1WIRE_DATA+2+13)
	STS(adr_DRUCK_DIGITAL_H),temp
	LDS temp,(adr_1WIRE_DATA+2+14)		; Luftdruck analog			
	STS(adr_DRUCK_ANALOG_L),temp
	LDS temp,(adr_1WIRE_DATA+2+15)
	STS(adr_DRUCK_ANALOG_H),temp
	LDS temp,(adr_1WIRE_DATA+2+16) 		;(16) Aussentemperatur			
	STS(adr_TEMPERATUR_AUSSEN_L),temp
	LDS temp,(adr_1WIRE_DATA+2+17)
	STS(adr_TEMPERATUR_AUSSEN_H),temp
;-------------------------------------------------------------------------------
	LDS temp,(adr_1WIRE_DATA+2+20)		; WZ-Raum
	STS(adr_TEMPERATUR_RAUM_WZ_L),temp
	LDS temp,(adr_1WIRE_DATA+2+21)
	STS(adr_TEMPERATUR_RAUM_WZ_H),temp
	LDS temp,(adr_1WIRE_DATA+2+22)		; AZ-Raum
	STS(adr_TEMPERATUR_RAUM_AZ_L),temp
	LDS temp,(adr_1WIRE_DATA+2+23)
	STS(adr_TEMPERATUR_RAUM_AZ_H),temp
	LDS temp,(adr_1WIRE_DATA+2+24)		; K-Raum
	STS(adr_TEMPERATUR_RAUM_KU_L),temp
	LDS temp,(adr_1WIRE_DATA+2+25)
	STS(adr_TEMPERATUR_RAUM_KU_H),temp
	LDS temp,(adr_1WIRE_DATA+2+26)		; BA-Raum
	STS(adr_TEMPERATUR_RAUM_BA_L),temp
	LDS temp,(adr_1WIRE_DATA+2+27)
	STS(adr_TEMPERATUR_RAUM_BA_H),temp
	LDS temp,(adr_1WIRE_DATA+2+28)		; FL-Raum
	STS(adr_TEMPERATUR_RAUM_FL_L),temp
	LDS temp,(adr_1WIRE_DATA+2+29)
	STS(adr_TEMPERATUR_RAUM_FL_H),temp
	LDS temp,(adr_1WIRE_DATA+2+30)		; TH-Raum
	STS(adr_TEMPERATUR_RAUM_TH_L),temp
	LDS temp,(adr_1WIRE_DATA+2+31)
	STS(adr_TEMPERATUR_RAUM_TH_H),temp
;-------------------------------------------------------------------------------
	LDS temp,(adr_1WIRE_DATA+2+40)		; WZ-HZG 
	STS(adr_TEMPERATUR_HZG_WZ_L),temp
	LDS temp,(adr_1WIRE_DATA+2+41)
	STS(adr_TEMPERATUR_HZG_WZ_H),temp
	LDS temp,(adr_1WIRE_DATA+2+42)		; AZ-HZG 
	STS(adr_TEMPERATUR_HZG_AZ_L),temp
	LDS temp,(adr_1WIRE_DATA+2+43)
	STS(adr_TEMPERATUR_HZG_AZ_H),temp
	LDS temp,(adr_1WIRE_DATA+2+44)		; K-HZG 
	STS(adr_TEMPERATUR_HZG_KU_L),temp
	LDS temp,(adr_1WIRE_DATA+2+45)
	STS(adr_TEMPERATUR_HZG_KU_H),temp
	LDS temp,(adr_1WIRE_DATA+2+46)		; BA-HZG 
	STS(adr_TEMPERATUR_HZG_BA_L),temp
	LDS temp,(adr_1WIRE_DATA+2+47)
	STS(adr_TEMPERATUR_HZG_BA_H),temp
;-------------------------------------------------------------------------------
	LDS temp,(adr_1WIRE_DATA+2+50)		; HAUS KW
	STS(adr_TEMPERATUR_HAUS_KW_L),temp
	LDS temp,(adr_1WIRE_DATA+2+51)
	STS(adr_TEMPERATUR_HAUS_KW_H),temp
	LDS temp,(adr_1WIRE_DATA+2+52)		; HAUS WW
	STS(adr_TEMPERATUR_HAUS_WW_L),temp
	LDS temp,(adr_1WIRE_DATA+2+53)
	STS(adr_TEMPERATUR_HAUS_WW_H),temp
	LDS temp,(adr_1WIRE_DATA+2+54)		; HAUS HZG
	STS(adr_TEMPERATUR_HAUS_HZG_L),temp
	LDS temp,(adr_1WIRE_DATA+2+55)
	STS(adr_TEMPERATUR_HAUS_HZG_H),temp
;-------------------------------------------------------------------------------
	LDS temp,(adr_1WIRE_DATA+2+60)
	STS(adr_RESERVE0_L),temp			; RES-0
	LDS temp,(adr_1WIRE_DATA+2+61)
	STS(adr_RESERVE0_H),temp
	LDS temp,(adr_1WIRE_DATA+2+62)		
	STS(adr_RESERVE1_L),temp			; RES-1
	LDS temp,(adr_1WIRE_DATA+2+63)
	STS(adr_RESERVE1_H),temp
	LDS temp,(adr_1WIRE_DATA+2+64)
	STS(adr_RESERVE2_L),temp			; RES-2
	LDS temp,(adr_1WIRE_DATA+2+65)
	STS(adr_RESERVE2_H),temp
	LDS temp,(adr_1WIRE_DATA+2+66)
	STS(adr_RESERVE3_L),temp			; RES-3
	LDS temp,(adr_1WIRE_DATA+2+67)
	STS(adr_RESERVE3_H),temp
	LDS temp,(adr_1WIRE_DATA+2+68)
	STS(adr_RESERVE4_L),temp			; RES-4
	LDS temp,(adr_1WIRE_DATA+2+69)
	STS(adr_RESERVE4_H),temp
;-------------------------------------------------------------------------------
	LDS temp,(adr_1WIRE_DATA+2+70)
	STS(adr_RF_AUSSEN),temp				; % Aussen	
	LDS temp,(adr_1WIRE_DATA+2+71)
	STS(adr_RF_WZ),temp					; % WZ		
	LDS temp,(adr_1WIRE_DATA+2+72)
	STS(adr_RF_AZ),temp					; % AZ		
	LDS temp,(adr_1WIRE_DATA+2+73)
	STS(adr_RF_KU),temp					; % K		
	LDS temp,(adr_1WIRE_DATA+2+74)
	STS(adr_RF_BA),temp					; % BA		

	LDS temp,(adr_1WIRE_DATA+2+75)
	STS(adr_RESERVE_A),temp				; Reserve-A	
	LDS temp,(adr_1WIRE_DATA+2+76)
	STS(adr_RESERVE_B),temp				; Reserve-B	
	LDS temp,(adr_1WIRE_DATA+2+77)
	STS(adr_RESERVE_C),temp				; Reserve-C	
	LDS temp,(adr_1WIRE_DATA+2+78)
	STS(adr_RESERVE_D),temp				; Reserve-D	
	LDS temp,(adr_1WIRE_DATA+2+79)
	STS(adr_RESERVE_E),temp				; Reserve-E	
;-------------------------------------------------------------------------------
	LDS temp,(adr_1WIRE_DATA+2+80)
	STS(adr_ANZAHL_KLINGEL),temp			; Klingel		
	LDS temp,(adr_1WIRE_DATA+2+81)
	STS(adr_ANZAHL_ANRUFE),temp				; Anrufe		
	LDS temp,(adr_1WIRE_DATA+2+82)
	STS(adr_ENERGIESPAR_MODUS),temp			; ENERGIESPAR	

	LDS temp,(adr_1WIRE_DATA+2+83)
	STS(adr_NIEDERSCHLAG),temp				; NIEDERSCHLAG	

	LDS temp,(adr_1WIRE_DATA+2+84)
	STS(adr_NIESELREGEN),temp				; NIESELREGEN	

	LDS temp,(adr_1WIRE_DATA+2+85)
	STS(adr_GEWITTER_PRO_STUNDE),temp		; GEWITTER pro h

	LDS temp,(adr_1WIRE_DATA+2+86)
	STS(adr_GEWITTER_PRO_TAG),temp			; GEWITTER pro Tag

	LDS temp,(adr_1WIRE_DATA+2+87)
	STS(adr_RADIOAKTIVITAET),temp			; RADIOAKTIVITAET

	LDS temp,(adr_1WIRE_DATA+2+88)
	STS(adr_SONNE),temp						; SONNE			

	LDS temp,(adr_1WIRE_DATA+2+89)
	STS(adr_WINDSTAERKE),temp				; WINDSTAERKE	

	LDS temp,(adr_1WIRE_DATA+2+90)
	STS(adr_WINDRICHTUNG),temp				; WINDRICHTUNG	


	LDS temp,(adr_1WIRE_DATA+2+91)
	LDS temp,(adr_1WIRE_DATA+2+92)
	LDS temp,(adr_1WIRE_DATA+2+93)
	LDS temp,(adr_1WIRE_DATA+2+94)
	LDS temp,(adr_1WIRE_DATA+2+95)
	LDS temp,(adr_1WIRE_DATA+2+96)

	LDS temp,(adr_1WIRE_DATA+2+97)
	STS(adr_OW_MASTER_WARNUNG_ANZAHL),temp	; WARNUNG		
	LDS temp,(adr_1WIRE_DATA+2+98)
 	STS(adr_OW_MASTER_ERROR_ANZAHL),temp	; Error			
;-------------------------------------------------------------------------------

	LDS temp,(adr_1WIRE_DATA+2+99)			; PRFSUMME		
	STS(adr_1WIRE_PS_EMPFANGEN),temp

	rjmp OWIRE_RX_SPERREN					; RX sperren	
; ##############################################################################
; ##############################################################################
; ##############################################################################
OWIRE_RX_SPERREN:
	STS(adr_1WIRE_STATUS),NULL
	STS(adr_1WIRE_BIT_COUNTER),NULL
	STS(adr_1WIRE_BYTE_COUNTER),NULL
	rjmp OW_PRESENCE_CHECK				; RP-Check				
; ##############################################################################
; ##############################################################################
; ##############################################################################
OW_PRESENCE_CHECK:
	clr ZL
	clr ZH
OW_RESET_PRESENCE_s:
	SBIC(OWIRE_PIN),(OWIRE_PIN_NR)		; Sprung, wenn PIN LOW
	rjmp OW_RESET_PRESENCE_AUSWERTUNG
	WDR						; 1us Pause				
	adiw ZL,1
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	rjmp OW_RESET_PRESENCE_s
;-------------------------------------------------------------------------------
OW_RESET_PRESENCE_AUSWERTUNG:
	cpi ZH,HIGH(400)
	breq OW_RESET_PRESENCE_L
	brsh OW_RESET_PRESENCE_ERKANNT
	ret
OW_RESET_PRESENCE_L:
	cpi ZL,LOW(400)
	brsh OW_RESET_PRESENCE_ERKANNT
	ret
;-------------------------------------------------------------------------------
OW_RESET_PRESENCE_ERKANNT:
	STS(adr_OW_ZAEHLER_RP_L),ZL
	STS(adr_OW_ZAEHLER_RP_H),ZH
	call WAIT_10us
	call WAIT_10us
	call WAIT_10us
	call WAIT_10us
	call WAIT_10us
	call WAIT_10us
	call WAIT_10us
;-------------------------------------------------------------------------------
; Warten, bis PIN wieder HIGH wird												
OWIRE_WAIT_RESET_PRESENCE_w:
	WDR
	SBIS(OWIRE_PIN),(OWIRE_PIN_NR)		; Sprung, wenn PIN HIGH
	rjmp OWIRE_WAIT_RESET_PRESENCE_w
;-------------------------------------------------------------------------------
	STS(adr_1WIRE_STATUS),EINS			
	STS(adr_1WIRE_BIT_COUNTER),NULL
	STS(adr_1WIRE_BYTE_COUNTER),NULL
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################






